/*
 * Mach Operating System
 * Copyright (c) 1991 Carnegie-Mellon University
 * All rights reserved.  The CMU software License Agreement specifies
 * the terms and conditions for use and redistribution.
 */
/*
 * HISTORY
 * $Log:	bcopy.s,v $
 * Revision 2.2  91/09/04  11:28:57  jsb
 * 	First checkin. Provided by Intel SSD.
 * 	[91/09/04  10:32:49  jsb]
 * 
 */
/*
 * bcopy is in bzero.s. This file is provided because the Makefile looks
 * for it.
 */
