
typedef union  {
	char	*str;
	int	val;
	struct	file_list *file;
	struct	idlst *lst;
} YYSTYPE;
extern YYSTYPE yylval;
# define AND 257
# define ANY 258
# define ARGS 259
# define AT 260
# define BIO 261
# define COMMA 262
# define CONFIG 263
# define CONTROLLER 264
# define CPU 265
# define CSR 266
# define DEVICE 267
# define DISK 268
# define DRIVE 269
# define DRQ 270
# define DST 271
# define DUMPS 272
# define EQUALS 273
# define FLAGS 274
# define HZ 275
# define IDENT 276
# define IOMEM 277
# define IOSIZ 278
# define IRQ 279
# define MACHINE 280
# define MAJOR 281
# define MASTER 282
# define MAXUSERS 283
# define MINOR 284
# define MINUS 285
# define NET 286
# define NEXUS 287
# define ON 288
# define OPTIONS 289
# define MAKEOPTIONS 290
# define PORT 291
# define PRIORITY 292
# define PSEUDO_DEVICE 293
# define ROOT 294
# define SEMICOLON 295
# define SIZE 296
# define SLAVE 297
# define SWAP 298
# define TIMEZONE 299
# define TTY 300
# define TRACE 301
# define VECTOR 302
# define ID 303
# define NUMBER 304
# define FPNUMBER 305
